﻿// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

// -------------------------------------------------------------------
// FVX Sample Program
// 
//
// 作成者 : R.Nagashima
// 履歴   : 2007/01/17 初版
//
//-------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using FVDATASLib;


namespace Edge1
{
    public partial class frmMain : Form
    {
        public frmMain()
        {
            InitializeComponent();
        }

        // ビデオデバイスがオープンされているか
        private bool m_blnVideoOpened = false;
        // ビデオ設定ファイル名（フルパス）
        private string m_strVideoConfFilePath = "";
        // 初期ディレクトリ
        private string m_strDirectory;

        //エッジ検出線
        private FvLineSegment m_EdgeScanLine = new FvLineSegment();

        private void frmMain_Load(object sender, EventArgs e)
        {
            int IntErr;
            frmSelectvideoConfFile fVideoConf = new frmSelectvideoConfFile();
            frmLicense fLicense = new frmLicense();

            //コントロールの設定
            cmbEdgeColor.Items.Insert(0,"白→黒"); //HITE_TO_BLACK
            cmbEdgeColor.Items.Insert(1,"黒→白"); //BLACK_TO_WHITE
            cmbEdgeColor.Items.Insert(2,"無極性"); //ALL_COLOR
            cmbEdgeColor.SelectedIndex = 2;

            cmbEdgeStartPos.Items.Insert(0, "始点→終点"); //FROM_START 始点位置からエッジ走査します｡
            cmbEdgeStartPos.Items.Insert(1, "終点→始点"); //FROM_END 終点位置からエッジ走査します｡
            cmbEdgeStartPos.SelectedIndex = 0;
                        
            cmbEdgePoint.Items.Insert(0, "最大点");   //POINT_PEAK 微分値が最大の位置をエッジとします｡
		    cmbEdgePoint.Items.Insert(1, "最初の点"); //POINT_1ST しきい値を一番最初に越えた位置をエッジとします｡
            cmbEdgePoint.SelectedIndex = 0;

            // 変数初期化
            m_blnVideoOpened = false;
            btnGrab.Enabled = false;

             //画面の生成
            axFvxDisplayEx1.CreateDisplay();

            //エッジ検出線の初期値設定
            m_EdgeScanLine.xs = 200;
            m_EdgeScanLine.ys = 230;
            m_EdgeScanLine.xe = 400;
            m_EdgeScanLine.ye = 230;

            //作業ディレクトリ退避
            m_strDirectory = Directory.GetCurrentDirectory();

            //使用許諾の確認
            if (fLicense.ShowDialog() == DialogResult.OK)
            {
                //ビデオ設定ファイルの選択
                if (fVideoConf.ShowDialog() == DialogResult.OK)
                    m_strVideoConfFilePath = fVideoConf.strVideoConfFilePath;

                if (fVideoConf.chkNotUseCamera.Checked == false)
                {
                    //ビデオ設定ファイルをコンポーネントに設定
                    axFvxVideo1.VideoConfFile = m_strVideoConfFilePath;

                    //デバイスオープン
                    try
                    {
                        axFvxVideo1.OpenDevice();
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオデバイスのオープンに失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //VideoConfFileの変更後、OpenDeviceを行うと、トリガーモードの変更が伴います
                    //このため、弊社キャプチャボードFVC02では350msのセットアップタイムが必要になります｡
                    //本プログラムでは簡略化のために、一律に350msのセットアップタイムを確保していますが、
                    //お使いのキャプチャボードに合わせて変更してください。
                    Thread.Sleep(350);

                    //GrabSync/GrabASyncメソッドではImegeサイズの変更を行いませんので、
                    //あらかじめChangeImageSizeメソッドにてImageのサイズを変更してください
                    //（取り込みサイズとImageのサイズが違う場合エラーになります）
                    axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());
                    try
                    {
                        // 同期取込
                        axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                    }
                    catch (Exception)
                    {
                        axFvxVideo1.GetErrReport(out IntErr);
                        MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString() + "\r\n画像ファイルのみの利用になります。");
                        return;
                    }

                    //ビデオデバイスの状態を保存
                    m_blnVideoOpened = true;
                    btnGrab.Enabled = true;
                }

                //画像を表示
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

                //エッジ検出線を描画
                axFvxDisplayEx1.PenColor = Color.Lime;
                axFvxDisplayEx1.DrawData(0, m_EdgeScanLine);
            }
            else
            {
                //終了
                Application.Exit();
                return;
            }
        }

        //終了時
        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (m_blnVideoOpened)
                axFvxVideo1.CloseDevice();
        }

        //カメラ取り込み
        private void btnGrab_Click(object sender, EventArgs e)
        {
            //画像ファイル読み込みによって、画像サイズが初期化時から
            //変更されている可能性があるため、ChangeImageSizeを行います。
            //画像サイズが変更される可能性が無い場合は、初期化時に一度だけ行えば問題有りません。
            axFvxVideo1.ChangeImageSize(axFvxImage1.GetOcx());

			try
            {
                //同期取込
                axFvxVideo1.GrabSync(axFvxImage1.GetOcx());
                //画像表示
                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
            catch (Exception)
            {
                int IntErr;
                axFvxVideo1.GetErrReport(out IntErr);
                MessageBox.Show("ビデオの取込に失敗しました｡ ErrCode= " + IntErr.ToString());
            }    
        }

        //画像ファイル読込
        private void btnLoadImage_Click(object sender, EventArgs e)
        {
            // ダイアログにて画像ファイルを選択します｡
            openFileDialog1.InitialDirectory = m_strDirectory;
            openFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            openFileDialog1.DefaultExt = "*.bmp";
            openFileDialog1.ShowDialog();

            if (openFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを用いて、
                //ファイルからImageに画像を読み込みます｡
                axFvxFileAccess1.FilePath = openFileDialog1.FileName;

                try
                {
                    axFvxFileAccess1.LoadBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("読込みに失敗しました｡ ErrCode= " + IntErr.ToString());
                    return;
                }

                //画像タイプチェック
                if (axFvxImage1.ImageType != FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray || axFvxImage1.BitPerPixel != 8)
                {
                    MessageBox.Show("グレイ画像ではありません｡", "error", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                    axFvxImage1.BitPerPixel = 8;
                    axFvxImage1.ImageType = FVXCOMIMAGELib.FVXIMAGE_IMAGE_TYPE.imageGray;
                    axFvxImage1.ImageWidth = 512;
                    axFvxImage1.ImageHeight = 480;
                }

                axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
            }
        }

        //画像ファイル保存
        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            saveFileDialog1.InitialDirectory = m_strDirectory;
            saveFileDialog1.Filter = "bitmap file(*.bmp)|*.bmp";
            saveFileDialog1.DefaultExt = "*.bmp";
            saveFileDialog1.ShowDialog();

            if (saveFileDialog1.FileName.Length > 0)
            {
                //FileAccessコンポーネントを使って
                //Imageをファイルに保存します｡
                axFvxFileAccess1.FilePath = saveFileDialog1.FileName;
                try
                {
                    axFvxFileAccess1.SaveBitmap(axFvxImage1.GetOcx());
                }
                catch (Exception)
                {
                    int IntErr = 0;
                    axFvxFileAccess1.GetErrReport(out IntErr);
                    MessageBox.Show("保存に失敗しました。　ErrCode= " + IntErr.ToString());
                    return;
                }
            }
        }

        //検出線設定
        private void btnSetEdgeScanLine_Click(object sender, EventArgs e)
        {
            int IntOK = 0;

            //設定ダイアログの初期値を決めておきます。
            axFvxROI1.SetLineSegment(m_EdgeScanLine);

            //FvxROIを使って検出線選択ダイアログを出します。
            axFvxROI1.UseLineSegment = true;
            axFvxROI1.UsePoint = false;
            axFvxROI1.UseRectangle = false;

            axFvxROI1.MakeROIDialog(axFvxImage1.GetOcx(), out IntOK);

            if ((1 == IntOK) && (axFvxROI1.CheckLineSegment == 1))
            {
                axFvxROI1.GetLineSegment(m_EdgeScanLine);

                axFvxDisplayEx1.ClearOverlay(-1);

                axFvxDisplayEx1.PenColor = Color.Lime;
                axFvxDisplayEx1.DrawData(0, m_EdgeScanLine);
            }
        }

        //処理実行
        private void btnExecute_Click(object sender, EventArgs e)
        {
            
            FvEdgeResult EdgeResult = new FvEdgeResult();
            int intCount;//見つかったエッジ数
            double dblTime;//処理時間
                       
            
            //画面消去
            axFvxDisplayEx1.ClearOverlay(-1);

            try
            {
                //エッジ検出線を描画
                axFvxDisplayEx1.PenColor = Color.Lime;
                axFvxDisplayEx1.DrawData(0, m_EdgeScanLine);

                //エッジ閾値
                int Temp;
                if (int.TryParse(txtThreshold.Text, out Temp) == false || Temp < 0 || Temp > 200)//Ver2.40以降200まで使用可能
                {
                    MessageBox.Show("閾値は 0～200 の間で設定して下さい");
                    return;
                }
                axFvxEdge1.Thresh = Temp;

                //片幅
				if (int.TryParse(txtLineWidth.Text, out Temp) == false || Temp < 0)
                {
                    MessageBox.Show("片幅は 0以上で設定して下さい");
                    return;
                }
                else axFvxEdge1.LineWidth = Temp;

                //エッジ極性取得
                axFvxEdge1.EdgeColor = (FVXCOMEDGELib.EDGE_COLOR_TYPE)cmbEdgeColor.SelectedIndex;

                //検出方向取得
                axFvxEdge1.StartPosition = (FVXCOMEDGELib.START_POSITION_TYPE)cmbEdgeStartPos.SelectedIndex;

                //エッジ位置取得
                axFvxEdge1.EdgePoint = (FVXCOMEDGELib.EDGE_POINT_TYPE)cmbEdgePoint.SelectedIndex;

                axFvxPerformanceCounter1.StartCount();
                //エッジ検出実行
                axFvxEdge1.Execute(axFvxImage1.GetOcx(), m_EdgeScanLine, out intCount);

                axFvxPerformanceCounter1.StopCount(out dblTime);

                if (0 < intCount)
                {
                    //エッジ点を赤で表示
                    axFvxDisplayEx1.PenColor = Color.Red;

                    //エッジは1点のみなのでindexnoは0
                    axFvxEdge1.GetResult(0, EdgeResult);
                    axFvxDisplayEx1.DrawData(0, EdgeResult);

                    txtResult.Text = string.Format("処理時間 ： {0:0.00ms}\r\nエッジ点座標 ( {1:0.0},  {2:0.0} )", dblTime, EdgeResult.x, EdgeResult.y);

                }
                else MessageBox.Show("エッジが見つかりませんでした");
            }
            //COMエラーを取得
            catch (System.Runtime.InteropServices.COMException)
            {
                int ErrNum = 0;
                axFvxEdge1.GetErrReport(out ErrNum);
                MessageBox.Show( "errornum = " + ErrNum.ToString(), "エッジ計測エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            //システムエラーを取得
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //数値/Delete/BackSpaceのみ入力できるよう制御
        private void txtThreshold_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;
        }
        private void txtLineWidth_KeyPress(object sender, KeyPressEventArgs e)
        {
            if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
                e.Handled = true;

        }  
    }
}